"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var MermaidPlugin_1;
Object.defineProperty(exports, "__esModule", { value: true });
const converter_1 = require("typedoc/dist/lib/converter");
const components_1 = require("typedoc/dist/lib/converter/components");
const events_1 = require("typedoc/dist/lib/output/events");
let MermaidPlugin = MermaidPlugin_1 = class MermaidPlugin extends components_1.ConverterComponent {
    constructor() {
        super(...arguments);
        this.BODY_CLOSING_TAG = /<\/body>/;
    }
    static filterComment(comment) {
        return comment !== undefined && !!comment;
    }
    static filterCommentTags(tags) {
        return tags !== undefined && !!tags;
    }
    static isMermaidCommentTag(tag) {
        return tag.tagName === 'mermaid';
    }
    static mermaidTags(context) {
        return Object.values(context.project.reflections)
            .map(reflection => reflection.comment)
            .filter(this.filterComment)
            .map(comment => comment.tags)
            .filter(this.filterCommentTags)
            .reduce((a, b) => a.concat(b), [])
            .filter(this.isMermaidCommentTag);
    }
    convertCommentTagText(tagText) {
        const texts = tagText.split('\n');
        const title = texts.shift();
        const mermaid = texts.join('\n');
        return `#### ${title} \n\n <div class="mermaid">${mermaid}</div>`;
    }
    convertPageContents(contents) {
        if (this.BODY_CLOSING_TAG.test(contents)) {
            return contents.replace(this.BODY_CLOSING_TAG, MermaidPlugin_1.customScriptsAndBodyClosingTag);
        }
        return contents;
    }
    initialize() {
        this.listenTo(this.owner, {
            [converter_1.Converter.EVENT_RESOLVE_BEGIN]: this.onResolveBegin,
        }).listenTo(this.application.renderer, {
            [events_1.PageEvent.END]: this.onPageEnd,
        });
    }
    onResolveBegin(context) {
        MermaidPlugin_1.mermaidTags(context).forEach(tag => {
            tag.text = this.convertCommentTagText(tag.text);
        });
    }
    onPageEnd(page) {
        if (page.contents !== undefined) {
            page.contents = this.convertPageContents(page.contents);
        }
    }
};
MermaidPlugin.customScriptsAndBodyClosingTag = `
    <script
      src="https://unpkg.com/mermaid/dist/mermaid.min.js"
    ></script>
    <script>
    mermaid.initialize({
      startOnLoad: true,
    });
    </script>
    </body>
  `;
MermaidPlugin = MermaidPlugin_1 = __decorate([
    components_1.Component({ name: 'mermaid' })
], MermaidPlugin);
exports.MermaidPlugin = MermaidPlugin;
